# Copyright DataStax, Inc.
#
# Licensed under the DataStax DSE Driver License;
# you may not use this file except in compliance with the License.
#
# You may obtain a copy of the License at
#
# http://www.datastax.com/terms/datastax-dse-driver-license-terms

import os
import json
import json
import tempfile
import shutil
from six.moves.urllib.request import urlopen
from ssl import SSLContext, PROTOCOL_TLSv1, CERT_REQUIRED
from zipfile import ZipFile
try:
    from zipfile import BadZipFile
except:
    from zipfile import BadZipfile as BadZipFile

from dse import DriverException


class DbaasConfig(object):

    username = None
    password = None
    host = None
    port = None
    keyspace = None
    local_dc = None
    ssl_context = None

    sni_host = None
    sni_port = None
    host_ids = None

    @classmethod
    def from_dict(cls, d):
        c = cls()

        c.port = d.get('port', None)
        try:
            c.port = int(d['port'])
        except:
            pass

        c.username = d.get('username', None)
        c.password = d.get('password', None)
        c.host = d.get('host', None)
        c.keyspace = d.get('keyspace', None)
        c.local_dc = d.get('localDC', None)

        return c


def get_config(cloud_config):
    try:
        secure_connect_bundle = cloud_config['secure_connect_bundle']
    except:
        raise ValueError("The cloud config doesn't have a secure_connect_bundle specified.")

    try:
        return config_from_zip(secure_connect_bundle)
    except BadZipFile:
        raise ValueError("Unable to open the zip file for the cloud config. Check your secure connect bundle.")


def config_from_zip(config_path):
    with ZipFile(config_path) as zipfile:
        base_dir = os.path.dirname(config_path)
        tmp_dir = tempfile.mkdtemp(dir=base_dir)
        try:
            zipfile.extractall(path=tmp_dir)
            config = config_from_json(os.path.join(tmp_dir, 'config.json'))
        finally:
            shutil.rmtree(tmp_dir)

        return config


def config_from_json(path):
    with open(path, 'r') as stream:
        data = json.loads(stream.read())

    config = DbaasConfig.from_dict(data)
    config_dir = os.path.dirname(path)

    # Load the ssl_context before we delete the temporary directory
    ca_cert_location = os.path.join(config_dir, 'ca.crt')
    cert_location = os.path.join(config_dir, 'cert')
    key_location = os.path.join(config_dir, 'key')
    config.ssl_context = _ssl_context_from_cert(ca_cert_location, cert_location, key_location)

    return config_from_metadata(config)


def config_from_metadata(config):
    url = "https://{}:{}/metadata".format(config.host, config.port)
    response = urlopen(url, context=config.ssl_context)
    if response.code != 200:
        # TODO print error
        raise DriverException("Error while fetching cluster metadata at: %s" % url)
    return parse_metadata_config(config, response.read().decode('utf-8'))


def parse_metadata_config(config, http_data):
    try:
        data = json.loads(http_data)
    except:
        msg = "Failed to load cluster metadata"
        raise DriverException(msg)

    contact_info = data['contact_info']
    config.local_dc = contact_info['local_dc']

    proxy_info = contact_info['sni_proxy_address'].split(':')
    config.sni_host = proxy_info[0]
    try:
        config.sni_port = int(proxy_info[1])
    except:
        config.sni_port = 9042

    config.host_ids = [host_id for host_id in contact_info['contact_points']]

    return config


def _ssl_context_from_cert(ca_cert_location, cert_location, key_location):
    ssl_context = SSLContext(PROTOCOL_TLSv1)
    ssl_context.load_verify_locations(ca_cert_location)
    ssl_context.verify_mode = CERT_REQUIRED
    ssl_context.load_cert_chain(certfile=cert_location, keyfile=key_location)

    return ssl_context
